@@ -38,6 +38,7 @@ class FlexMock
       @location = location
       @expected_args = nil
       @count_validators = []
+      @signature_validator = SignatureValidator.new(self)
       @count_validator_class = ExactCountValidator
       @actual_count = 0
       @return_value = nil
@@ -65,6 +66,9 @@ class FlexMock
       @count_validators.each do |validator|
         result << validator.describe
       end
+      if !@signature_validator.null?
+        result << @signature_validator.describe
+      end
       result
     end
 
@@ -79,11 +83,18 @@ class FlexMock
       FlexMock.framework_adapter.check(e.message) { false }
     end
 
+    def validate_signature(args)
+      @signature_validator.validate(args)
+    rescue SignatureValidator::ValidationFailed => e
+      FlexMock.framework_adapter.check(e.message) { false }
+    end
+
     # Verify the current call with the given arguments matches the
     # expectations recorded in this object.
     def verify_call(*args)
       validate_eligible
       validate_order
+      validate_signature(args)
       @actual_count += 1
       perform_yielding(args)
       return_value(args)
@@ -116,7 +127,9 @@ class FlexMock
         block = args.last
         values = (@yield_queue.size == 1) ? @yield_queue.first : @yield_queue.shift
         if block && block.respond_to?(:call)
-          @return_value = block.call(*values)
+          values.each do |v|
+            @return_value = block.call(*v)
+          end
         else
           fail MockError, "No Block given to mock with 'and_yield' expectation"
         end
@@ -130,11 +143,6 @@ class FlexMock
       @count_validators.all? { |v| v.eligible?(@actual_count) }
     end
 
-    # Is this expectation constrained by any call counts?
-    def call_count_constrained?
-      ! @count_validators.empty?
-    end
-
     # Validate that the order
     def validate_order
       if @order_number
@@ -184,6 +192,28 @@ class FlexMock
       self
     end
 
+    # Validate general parameters on the call signature
+    def with_signature(
+        required_arguments: 0, optional_arguments: 0, splat: false,
+        required_keyword_arguments: [], optional_keyword_arguments: [], keyword_splat: false)
+      @signature_validator = SignatureValidator.new(
+          self,
+          required_arguments: required_arguments,
+          optional_arguments: optional_arguments,
+          splat: splat,
+          required_keyword_arguments: required_keyword_arguments,
+          optional_keyword_arguments: optional_keyword_arguments,
+          keyword_splat: keyword_splat)
+      self
+    end
+
+    # Validate that the passed arguments match the method signature from the
+    # given instance method
+    def with_signature_matching(instance_method)
+      @signature_validator = SignatureValidator.from_instance_method(self, instance_method)
+      self
+    end
+
     # :call-seq:
     #   and_return(value)
     #   and_return(value, value, ...)
@@ -251,10 +281,30 @@ class FlexMock
     # An error is raised if the mocked method is not called with a
     # block.
     def and_yield(*yield_values)
-      @yield_queue << yield_values
+      @yield_queue << [yield_values]
     end
     alias :yields :and_yield
 
+    # Declare that the mocked method is expected to be given a block
+    # and that the block will iterate over the provided values.
+    # If the mock is called multiple times, mulitple
+    # <tt>and_iterates</tt> declarations can be used to supply different
+    # values on each call.
+    #
+    # The iteration is queued with the yield values provided with {#and_yield}.
+    #
+    # An error is raised if the mocked method is not called with a
+    # block.
+    #
+    # @example interaction of and_yield and and_iterates
+    #   mock.should_receive(:each).and_yield(10).and_iterates(1, 2, 3).and_yield(20)
+    #   mock.enum_for(:each).to_a # => [10]
+    #   mock.enum_for(:each).to_a # => [1,2,3]
+    #   mock.enum_for(:each).to_a # => [20]
+    #
+    def and_iterates(*yield_values)
+      @yield_queue << yield_values
+    end
 
     # :call-seq:
     #   and_raise(an_exception)
