@@ -28,46 +28,92 @@ class FlexMock
 
     attr_reader :mock
 
-    ProxyBox = Struct.new :proxy
+    # Boxing of the flexmock proxy
+    #
+    # It is managed as a stack in order to allow to setup containers recursively
+    # (as e.g. FlexMock.use( ... ) checks)
+    class ProxyBox
+      attr_reader :stack
+
+      Element = Struct.new :proxy, :container
+
+      def initialize
+        @stack = [Element.new]
+      end
+
+      # Tests whether the given container is the one on which the current proxy
+      # acts
+      def container
+        stack.last.container
+      end
+
+      def proxy
+        stack.last.proxy
+      end
+
+      def push(proxy, container)
+        stack.push(Element.new(proxy, container))
+      end
+
+      def pop
+        if !stack.empty?
+          stack.pop
+        end
+      end
+
+      def empty?
+        stack.size == 1
+      end
+    end
 
     # Make a partial mock proxy and install it on the target +obj+.
     def self.make_proxy_for(obj, container, name, safe_mode)
       name ||= "flexmock(#{obj.class.to_s})"
-      if ! proxy_defined_on?(obj)
-        mock = FlexMock.new(name, container)
-        proxy = PartialMockProxy.new(obj, mock, safe_mode)
-        if obj.instance_variable_defined?("@flexmock_proxy")
-          obj.instance_variable_get("@flexmock_proxy").proxy = proxy
-        else
-          obj.instance_variable_set("@flexmock_proxy", ProxyBox.new(proxy))
-        end
+      if !obj.instance_variable_defined?("@flexmock_proxy")
+        proxy_box = obj.instance_variable_set("@flexmock_proxy", ProxyBox.new)
+      else
+        proxy_box = obj.instance_variable_get("@flexmock_proxy")
       end
-      obj.instance_variable_get("@flexmock_proxy").proxy
-    end
 
-    # Is there a mock proxy defined on the domain object?
-    def self.proxy_defined_on?(obj)
-      obj.instance_variable_defined?("@flexmock_proxy") &&
-        obj.instance_variable_get("@flexmock_proxy").proxy
+      if proxy_box.container != container
+        if !proxy_box.empty?
+          parent_proxy, _ = proxy_box.proxy
+          parent_mock = parent_proxy.mock
+        end
+
+        mock  = FlexMock.new(name, container, parent: parent_mock)
+        proxy = PartialMockProxy.new(obj, mock, safe_mode, parent: parent_proxy)
+        proxy_box.push(proxy, container)
+      end
+      proxy_box.proxy
     end
 
     # The following methods are added to partial mocks so that they
     # can act like a mock.
 
     MOCK_METHODS = [
-      :should_receive, :new_instances,
+      :should_receive, :new_instances, :should_expect,
       :should_receive_with_location,
       :flexmock_get,   :flexmock_teardown, :flexmock_verify,
-      :flexmock_received?, :flexmock_calls, :flexmock_find_expectation
+      :flexmock_received?, :flexmock_calls, :flexmock_find_expectation,
+      :invoke_original
     ]
 
+    attr_reader :method_definitions
+
     # Initialize a PartialMockProxy object.
-    def initialize(obj, mock, safe_mode)
+    def initialize(obj, mock, safe_mode, parent: nil)
       @obj = obj
       @mock = mock
-      @method_definitions = {}
-      @methods_proxied = []
       @proxy_definition_module = nil
+      @initialize_override = nil
+
+      if parent
+        @method_definitions = parent.method_definitions.dup
+      else
+        @method_definitions = {}
+      end
+
       unless safe_mode
         add_mock_method(:should_receive)
         MOCK_METHODS.each do |sym|
@@ -83,6 +129,14 @@ class FlexMock
       @mock
     end
 
+    def push_flexmock_container(container)
+      @mock.push_flexmock_container(container)
+    end
+
+    def pop_flexmock_container
+      @mock.pop_flexmock_container
+    end
+
     # :call-seq:
     #    should_receive(:method_name)
     #    should_receive(:method1, method2, ...)
@@ -107,12 +161,42 @@ class FlexMock
       flexmock_define_expectation(caller, *args)
     end
 
+    def should_expect(*args)
+      yield Recorder.new(self)
+    end
+
+    # Invoke the original of a mocked method
+    #
+    # Usually called in a #and_return statement
+    def invoke_original(m, *args, &block)
+      if block
+        args << block
+      end
+      flexmock_invoke_original(m, args)
+    end
+
+    # Whether the given method's original definition has been stored
+    def has_original_method?(m)
+      @method_definitions.has_key?(m)
+    end
+
+    # Whether the given method is already being proxied
+    def has_proxied_method?(m)
+      @proxy_definition_module &&
+          @proxy_definition_module.method_defined?(m)
+    end
+
     def flexmock_define_expectation(location, *args)
       EXP_BUILDER.parse_should_args(self, args) do |method_name|
-        unless @methods_proxied.include?(method_name)
+        if !has_proxied_method?(method_name) && !has_original_method?(method_name)
           hide_existing_method(method_name)
         end
         ex = @mock.flexmock_define_expectation(location, method_name)
+        if FlexMock.partials_verify_signatures
+          if existing_method = @method_definitions[method_name]
+            ex.with_signature_matching(existing_method)
+          end
+        end
         ex.mock = self
         ex
       end
@@ -156,8 +240,14 @@ class FlexMock
       fail ArgumentError, "new_instances requires a Class to stub" unless
         Class === @obj
       location = caller
-      allocators = [:new] if allocators.empty?
+      allocators = [:initialize] if allocators.empty?
+
       expectation_recorder = ExpectationRecorder.new
+
+      if allocators.delete(:initialize)
+        initialize_stub(expectation_recorder, block)
+      end
+
       allocators.each do |allocate_method|
         flexmock_define_expectation(location, allocate_method).and_return { |*args|
           create_new_mocked_object(
@@ -167,6 +257,46 @@ class FlexMock
       expectation_recorder
     end
 
+    # Stubs the #initialize method on a class
+    def initialize_stub(recorder, expectations_block)
+      if !@initialize_override
+        expectation_blocks    = @initialize_expectation_blocks = Array.new
+        expectation_recorders = @initialize_expectation_recorders = Array.new
+        @initialize_override = Module.new do
+          define_method :initialize do |*args, &block|
+            if self.class.respond_to?(:__flexmock_proxy) && (mock = self.class.__flexmock_proxy)
+              container = mock.flexmock_container
+              mock = container.flexmock(self)
+              expectation_blocks.each do |b|
+                b.call(mock)
+              end
+              expectation_recorders.each do |r|
+                r.apply(mock)
+              end
+            end
+            super(*args, &block)
+          end
+        end
+        override = @initialize_override
+        @obj.class_eval { prepend override }
+      end
+      if expectations_block
+        @initialize_expectation_blocks    << expectations_block
+      end
+      @initialize_expectation_recorders << recorder
+    end
+
+    def initialize_stub?
+      !!@initialize_override
+    end
+
+    def initialize_stub_remove
+      if initialize_stub?
+        @initialize_expectation_blocks.clear
+        @initialize_expectation_recorders.clear
+      end
+    end
+
     # Create a new mocked object.
     #
     # The mocked object is created using the following steps:
@@ -206,6 +336,7 @@ class FlexMock
     # Remove all traces of the mocking framework from the existing object.
     def flexmock_teardown
       if ! detached?
+        initialize_stub_remove
         proxy_module_eval do
           methods = instance_methods(false).to_a
           methods.each do |m|
@@ -214,7 +345,7 @@ class FlexMock
         end
         if @obj.instance_variable_defined?(:@flexmock_proxy) &&
             (box = @obj.instance_variable_get(:@flexmock_proxy))
-          box.proxy = nil
+          box.pop
         end
         @obj = nil
       end
@@ -268,7 +399,7 @@ class FlexMock
       if !@proxy_definition_module
         obj = @obj
         @proxy_definition_module = m = Module.new do
-          define_method(:__flexmock_proxy) do
+          define_method("__flexmock_proxy") do
             if box = obj.instance_variable_get(:@flexmock_proxy)
               box.proxy
             end
@@ -286,17 +417,18 @@ class FlexMock
     # not a singleton, all we need to do is override it with our own
     # singleton.
     def hide_existing_method(method_name)
-      stow_existing_definition(method_name)
+      existing_method = stow_existing_definition(method_name)
       define_proxy_method(method_name)
+      existing_method
     end
 
     # Stow the existing method definition so that it can be recovered
     # later.
     def stow_existing_definition(method_name)
-      if !@methods_proxied.include?(method_name)
+      if !@method_definitions.has_key?(method_name)
         @method_definitions[method_name] = target_class_eval { instance_method(method_name) }
-        @methods_proxied << method_name
       end
+      @method_definitions[method_name]
     rescue NameError
     end
 
@@ -324,6 +456,5 @@ class FlexMock
     def detached?
       @obj.nil?
     end
-
   end
 end
