@@ -92,6 +92,23 @@ class TestStubbing < Minitest::Test
     assert_equal :woof, dog.bark
   end
 
+  def test_invoke_original_allows_to_call_the_original_directly
+    dog = Dog.new
+    flexmock(dog).should_receive(:bark).never
+    assert_equal :woof, dog.invoke_original(:bark)
+  end
+
+  def test_invoke_original_can_pass_a_block
+    dog = Class.new do
+      def bark(&block)
+        block.call
+      end
+    end.new
+    recorder = flexmock.should_receive(:called).once.mock
+    flexmock(dog).should_receive(:bark).never
+    dog.invoke_original(:bark) { recorder.called }
+  end
+
   def test_stubbed_methods_can_invoke_original_behavior_with_modification
     dog = Dog.new
     flexmock(dog).should_receive(:bark).pass_thru { |result| result.to_s.upcase }.once
@@ -122,7 +139,7 @@ class TestStubbing < Minitest::Test
     assert_equal :growl, dog.bark
     partial_mock.flexmock_teardown
     assert_equal :woof, dog.bark
-    assert_equal nil, dog.instance_variable_get("@flexmock_proxy").proxy
+    assert_nil dog.instance_variable_get("@flexmock_proxy").proxy
   end
 
   def test_original_missing_behavior_can_be_restored
@@ -433,6 +450,81 @@ class TestStubbing < Minitest::Test
     end
   end
 
+  def test_partial_mocks_properly_detect_methods_defined_through_a_class_hierarchy
+    dog = Class.new do
+      class << self
+        def bark
+        end
+      end
+    end
+    chiwawa = Class.new(dog)
+
+    FlexMock.partials_are_based = true
+    flexmock(chiwawa)
+    chiwawa.should_receive(:bark).and_return(:grrr)
+    assert_equal :grrr, chiwawa.bark
+  ensure
+    FlexMock.partials_are_based = false
+  end
+
+  if FlexMock::ON_RUBY_20
+    # This is a limitation due to 2.0's broken #ancestors on singletons of classes
+    def test_partial_mocks_will_not_require_explicitly_on_a_class_singleton_method_that_has_been_mocked_on_the_parent_class
+      dog = Class.new
+      chiwawa = Class.new(dog)
+
+      FlexMock.partials_are_based = true
+      flexmock(dog).should_receive(:bark).explicitly
+      flexmock(chiwawa)
+      chiwawa.should_receive(:bark).and_return(:grrr)
+      assert_equal :grrr, chiwawa.bark
+    ensure
+      FlexMock.partials_are_based = false
+    end
+  else
+    def test_based_partial_mocks_require_explicitly_on_a_non_existing_method_of_a_class_singleton
+      dog = Class.new
+      FlexMock.partials_are_based = true
+      assert_raises(NoMethodError, /bark.*explicitly/) do
+        flexmock(dog).should_receive(:bark).and_return(:grrr)
+      end
+    ensure
+      FlexMock.partials_are_based = false
+    end
+
+    def test_partial_mocks_require_explicitly_on_a_class_singleton_method_that_has_been_mocked_on_the_parent_class
+      dog = Class.new
+      chiwawa = Class.new(dog)
+
+      FlexMock.partials_are_based = true
+      flexmock(dog).should_receive(:bark).explicitly
+      flexmock(chiwawa)
+      assert_raises(NoMethodError, /bark.*explicitly/) do
+        chiwawa.should_receive(:bark).and_return(:grrr)
+      end
+    ensure
+      FlexMock.partials_are_based = false
+    end
+  end
+
+  def test_partial_mocks_do_not_stow_their_own_method_definitions
+    dog = Dog.new
+    flexmock(dog)
+    dog.should_receive(:meow).explicitly.and_return(:something)
+    dog.should_receive(:meow).explicitly.and_return(:something_else)
+    refute dog.__flexmock_proxy.has_original_method?(:meow)
+  end
+
+  def test_partial_mocks_allow_explicitely_stubbing_methods_in_sequence_when_signature_verification_is_on
+    dog = Dog.new
+    flexmock(dog, :on, Dog)
+    FlexMock.partials_verify_signatures = true
+    dog.should_receive(:meow).explicitly.and_return(:something)
+    dog.should_receive(:meow).explicitly.and_return(:something_else)
+  ensure
+    FlexMock.partials_verify_signatures = false
+  end
+
   # The following test was suggested by Pat Maddox for the RSpec
   # mocks.  Evidently the (poorly implemented) == method caused issues
   # with RSpec Mock's internals.  I'm just double checking for any
@@ -510,6 +602,29 @@ class TestStubbing < Minitest::Test
     assert(/pass_thru/ === exception.message, "expected #{exception.message} to mention the flexmock pass_thru clause")
   end
 
+  def test_partial_mocks_leaves_exceptions_raised_by_the_original_method_unchanged
+    error_m = Class.new(RuntimeError)
+    obj = Class.new do
+      define_method(:mocked_method) { raise error_m, "the error message" }
+    end.new
+    flexmock(obj).should_receive(:mocked_method).pass_thru
+    exception = assert_raises(error_m) do
+        obj.mocked_method
+    end
+    assert_equal "the error message", exception.message
+  end
+
+  def test_partial_mocks_leaves_NoMethodError_exceptions_raised_by_the_original_method_unchanged
+    obj = Class.new do
+      define_method(:mocked_method) { does_not_exist() }
+    end.new
+    flexmock(obj).should_receive(:mocked_method).pass_thru
+    exception = assert_raises(NameError) do
+        obj.mocked_method
+    end
+    assert_equal "undefined method `does_not_exist' for #{obj}", exception.message
+  end
+
   def test_it_checks_whether_mocks_are_forbidden_before_forwarding_the_call
       obj = Class.new
       flexmock(obj).should_receive(:mocked).never
@@ -540,4 +655,107 @@ class TestStubbing < Minitest::Test
         obj.inspect
       end
   end
+
+  def test_the_strict_keyword_raises_if_used_on_a_non_partial_mock
+    error = assert_raises(ArgumentError) do
+      flexmock('name', :strict)
+    end
+    assert_equal "cannot use :strict outside a partial mock", error.message
+  end
+
+  def test_the_strict_keyword_sets_the_base_class_to_the_partial_mock_singleton_class
+    klass = Class.new
+    m = Module.new
+    obj = klass.new
+    obj.extend m
+    mock = flexmock(obj, :strict)
+    assert_equal obj.singleton_class, mock.flexmock_get.flexmock_base_class
+  end
+
+  def test_it_verifies_the_signature_against_the_original_method_if_partials_verify_signatures_is_set
+    FlexMock.partials_verify_signatures = true
+    k = Class.new do
+      def m(a); end
+    end
+    flexmock(obj = k.new).should_receive(:m)
+
+    assert_mock_failure(check_failed_error, deep: true, message: /expects at least 1 positional arguments but got only 0/, line: __LINE__+1) do
+      obj.m
+    end
+  ensure
+    FlexMock.partials_verify_signatures = false
+  end
+
+  def test_it_can_setup_mocks_recursively
+    obj = Object.new
+    FlexMock.use(obj) do
+      obj.should_receive(:foo).once
+      FlexMock.use(obj) do
+        obj.should_receive(:blah).once
+        obj.blah
+      end
+      obj.should_receive(:bar).once
+      obj.foo
+      obj.bar
+    end
+  end
+
+  def test_it_passes_calls_from_child_contexts_to_parent_contexts
+    obj = Object.new
+    FlexMock.use(obj) do
+      obj.should_receive(:foo).once
+      FlexMock.use(obj) do
+        obj.foo
+      end
+    end
+  end
+
+  def test_expectations_defined_in_sub_contexts_are_added_to_the_ones_in_parent_contexts
+    obj = Object.new
+    FlexMock.use(obj) do
+      obj.should_receive(:foo).with(10).once
+      FlexMock.use(obj) do
+        obj.should_receive(:foo).with(20).once
+        obj.foo(10)
+        obj.foo(20)
+      end
+    end
+  end
+
+  def test_it_calls_the_true_original_method_in_children_contexts
+    obj = Class.new do
+      attr_reader :value
+      def initialize; @value = 0 end
+      def call(value); @value += 1 end
+    end.new
+
+    FlexMock.use(obj) do
+      obj.should_receive(:call).with(1)
+      FlexMock.use(obj) do
+        obj.should_receive(:call).with(2).pass_thru
+        obj.call(2)
+      end
+      obj.call(1)
+    end
+    assert_equal 1, obj.value
+  end
+
+  def test_should_expect
+    flexmock(obj = Dog.new).should_expect do |e|
+      e.bark
+    end
+    obj.bark
+  end
+
+  def test_interaction_between_signature_verification_and_based_partials
+    FlexMock.partials_are_based = true
+    FlexMock.partials_verify_signatures = true
+    obj = flexmock(obj = Dog.new)
+    obj.should_receive(:puts).explicitly.once
+    obj.puts "test"
+  ensure
+    FlexMock.partials_are_based = false
+    FlexMock.partials_verify_signatures = false
+  end
 end
+
