@@ -162,6 +162,29 @@ class TestFlexMockShoulds < Minitest::Test
     end
   end
 
+  def test_iteration_yields_values_in_sequence
+    FlexMock.use do |m|
+      m.should_receive(:msg).and_iterates(1, 2, 3)
+      yielded_values = []
+      m.msg { |a| yielded_values << a }
+      assert_equal [1, 2, 3], yielded_values
+    end
+  end
+
+  def test_iteration_and_yields_are_queued
+    FlexMock.use do |m|
+      m.should_receive(:msg).
+          and_yield(:one).
+          and_iterates(1, 2, 3).
+          and_yield(:two)
+      yielded_values = []
+      yielded_values << m.enum_for(:msg).to_a
+      yielded_values << m.enum_for(:msg).to_a
+      yielded_values << m.enum_for(:msg).to_a
+      assert_equal [[:one], [1, 2, 3], [:two]], yielded_values
+    end
+  end
+
   def test_failure_if_no_block_given
     FlexMock.use do |m|
       m.should_receive(:hi).and_yield(:one, :two).once
@@ -481,11 +504,11 @@ class TestFlexMockShoulds < Minitest::Test
 
   def test_arg_matching_with_class
     FlexMock.use do |m|
-      m.should_receive(:hi).with(Fixnum).returns(10)
+      m.should_receive(:hi).with(0.class).returns(10)
       m.should_receive(:hi).with(Object).returns(20)
 
       assert_equal 10, m.hi(319)
-      assert_equal 10, m.hi(Fixnum)
+      assert_equal 10, m.hi(0.class)
       assert_equal 20, m.hi("hi")
     end
   end
@@ -854,31 +877,31 @@ class TestFlexMockShoulds < Minitest::Test
   def test_global_ordering_message_includes_received_calls
     e = assert_mock_failure(check_failed_error, :message =>OUT_OF_ORDER_ERROR_MESSAGE, :deep => true, :line => __LINE__+6) do
       FlexMock.use 'm' do |m|
-        one = m.should_receive(:one).globally.ordered
-        two = m.should_receive(:two).globally.ordered
+        m.should_receive(:one).globally.ordered
+        m.should_receive(:two).globally.ordered
         m.one
         m.two
         m.one
       end
     end
-    assert_match /one\(\) matched by should_receive\(:one\)/, e.message
-    assert_match /two\(\) matched by should_receive\(:two\)/, e.message
-    assert_match /one\(\) matched by should_receive\(:one\)/, e.message
+    assert_match(/one\(\) matched by should_receive\(:one\)/, e.message)
+    assert_match(/two\(\) matched by should_receive\(:two\)/, e.message)
+    assert_match(/one\(\) matched by should_receive\(:one\)/, e.message)
   end
 
   def test_ordering_message_includes_received_calls
     e = assert_mock_failure(check_failed_error, :message =>OUT_OF_ORDER_ERROR_MESSAGE, :deep => true, :line => __LINE__+6) do
       FlexMock.use 'm' do |m|
-        one = m.should_receive(:one).ordered
-        two = m.should_receive(:two).ordered
+        m.should_receive(:one).ordered
+        m.should_receive(:two).ordered
         m.one
         m.two
         m.one
       end
     end
-    assert_match /one\(\) matched by should_receive\(:one\)/, e.message
-    assert_match /two\(\) matched by should_receive\(:two\)/, e.message
-    assert_match /one\(\) matched by should_receive\(:one\)/, e.message
+    assert_match(/one\(\) matched by should_receive\(:one\)/, e.message)
+    assert_match(/two\(\) matched by should_receive\(:two\)/, e.message)
+    assert_match(/one\(\) matched by should_receive\(:one\)/, e.message)
   end
 
   def test_explicit_ordering_mixed_with_implicit_ordering_should_not_overlap
@@ -1172,6 +1195,260 @@ class TestFlexMockShoulds < Minitest::Test
     assert_operator(:"`") { |m| m.`("command") } # `
   end
 
+  def test_with_signature_required_arguments
+    FlexMock.use do |mock|
+      mock.should_receive(:m).with_signature(required_arguments: 2)
+      assert_mock_failure(check_failed_error, message: /expects at least 2 positional arguments but got only 1/, line: __LINE__+1) do
+        mock.m(1)
+      end
+      mock.m(1, 2)
+      assert_mock_failure(check_failed_error, message: /expects at most 2 positional arguments but got 3/, line: __LINE__+1) do
+        mock.m(1, 2, 3)
+      end
+    end
+  end
+
+  def test_a_proc_argument_last_can_be_interpreted_as_positional_argument
+    FlexMock.use do |mock|
+      mock.should_receive(:m).with_signature(required_arguments: 2)
+      mock.m(1) { }
+      mock.m(1, 2) { }
+    end
+  end
+
+  def test_with_signature_optional_arguments
+    FlexMock.use do |mock|
+      mock.should_receive(:m).with_signature(required_arguments: 2, optional_arguments: 2)
+      assert_mock_failure(check_failed_error, message: /expects at least 2 positional arguments but got only 1/, line: __LINE__+1) do
+        mock.m(1)
+      end
+      mock.m(1, 2)
+      mock.m(1, 2, 3)
+      mock.m(1, 2, 3, 4)
+      assert_mock_failure(check_failed_error, message: /expects at most 4 positional arguments but got 5/, line: __LINE__+1) do
+        mock.m(1, 2, 3, 4, 5)
+      end
+    end
+  end
+
+  def test_it_interprets_a_hash_as_last_positional_argument_if_no_keyword_arguments_are_expected
+    FlexMock.use do |mock|
+      mock = flexmock
+      mock.should_receive(:m).with_signature(required_arguments: 2, optional_arguments: 2)
+      mock.m(1, 2, 3, test: 4)
+    end
+  end
+
+  def test_with_signature_splat_validates_required_arguments
+    FlexMock.use do |mock|
+      mock = flexmock
+      mock.should_receive(:m).with_signature(required_arguments: 2, optional_arguments: 2, splat: true)
+      assert_mock_failure(check_failed_error, message: /expects at least 2 positional arguments but got only 1/, line: __LINE__+1) do
+        mock.m(1)
+      end
+    end
+  end
+
+  def test_with_signature_splat_allows_an_arbitrary_number_of_extra_arguments
+    FlexMock.use do |mock|
+      mock = flexmock
+      mock.should_receive(:m).with_signature(required_arguments: 2, optional_arguments: 2, splat: true)
+      mock.m(1, 2)
+      mock.m(1, 2, 3)
+      mock.m(1, 2, 3, 4)
+    end
+  end
+
+  def test_with_signature_required_keyword_arguments
+    FlexMock.use do |mock|
+      mock = flexmock
+      mock.should_receive(:m).
+        with_signature(required_keyword_arguments: [:a, :b])
+      assert_mock_failure(check_failed_error, message: /missing required keyword arguments a/, line: __LINE__+1) do
+        mock.m(b: 10)
+      end
+      mock.m(a: 10, b: 20)
+      assert_mock_failure(check_failed_error, message: /given unexpected keyword argument c/, line: __LINE__+1) do
+        mock.m(a: 10, b: 10, c: 20)
+      end
+    end
+  end
+
+  def test_with_signature_optional_keyword_arguments
+    FlexMock.use do |mock|
+      mock.should_receive(:m).
+        with_signature(required_keyword_arguments: [:a, :b], optional_keyword_arguments: [:c, :d])
+      assert_mock_failure(check_failed_error, message: /missing required keyword arguments a/, line: __LINE__+1) do
+        mock.m(b: 10)
+      end
+      mock.m(a: 10, b: 20)
+      mock.m(a: 10, b: 20, c: 30)
+      mock.m(a: 10, b: 20, c: 30, d: 40)
+      assert_mock_failure(check_failed_error, message: /given unexpected keyword argument e/, line: __LINE__+1) do
+        mock.m(a: 10, b: 10, e: 42)
+      end
+    end
+  end
+
+  def test_with_signature_keyword_splat
+    FlexMock.use do |mock|
+      mock.should_receive(:m).
+        with_signature(
+          required_keyword_arguments: [:a, :b],
+          optional_keyword_arguments: [:c, :d],
+          keyword_splat: true)
+      assert_mock_failure(check_failed_error, message: /missing required keyword arguments a/, line: __LINE__+1) do
+        mock.m(b: 10)
+      end
+      mock.m(a: 10, b: 20)
+      mock.m(a: 10, b: 20, c: 30)
+      mock.m(a: 10, b: 20, c: 30, d: 40)
+      mock.m(a: 10, b: 10, e: 42)
+    end
+  end
+
+  def test_with_signature_raises_if_no_keywords_are_given
+    FlexMock.use do |mock|
+      mock.should_receive(:m).
+        with_signature(required_keyword_arguments: [:b], required_arguments: 1)
+      assert_mock_failure(check_failed_error, message: /expects keyword arguments but none were provided/, line: __LINE__+1) do
+        mock.m(10)
+      end
+    end
+  end
+
+  def test_with_signature_handles_getting_a_basicobject_as_last_object
+    FlexMock.use do |mock|
+      mock.should_receive(:m).
+          with_signature(optional_arguments: 1, required_keyword_arguments: [:b])
+      assert_mock_failure(check_failed_error, message: /expects keyword arguments but none were provided/, line: __LINE__+1) do
+        mock.m(BasicObject.new)
+      end
+    end
+  end
+  def test_with_signature_removes_the_keywords_from_the_position_arguments
+    FlexMock.use do |mock|
+      mock.should_receive(:m).
+        with_signature(required_keyword_arguments: [:b], required_arguments: 1)
+      mock.m(10, b: 10)
+    end
+
+    FlexMock.use do |mock|
+      mock.should_receive(:m).
+        with_signature(optional_keyword_arguments: [:b], required_arguments: 1)
+      mock.m(10, b: 10)
+    end
+
+    FlexMock.use do |mock|
+      mock.should_receive(:m).
+        with_signature(keyword_splat: true, required_arguments: 1)
+      mock.m(10, b: 10)
+    end
+  end
+
+  def test_signature_validator_understands_that_a_hash_can_be_used_for_both_keywords_and_positional_arguments
+      FlexMock.use do |mock|
+        mock.should_receive(:m).
+          with_signature(optional_keyword_arguments: [:b], required_arguments: 1)
+        mock.m(Hash.new)
+      end
+  end
+
+  def test_signature_validator_understands_that_a_proc_last_can_both_be_a_positional_parameter_and_a_block
+      FlexMock.use do |mock|
+        mock.should_receive(:m).with_signature(required_arguments: 1)
+        mock.m(proc { })
+      end
+
+      FlexMock.use do |mock|
+        mock.should_receive(:m).with_signature(required_arguments: 0)
+        mock.m { }
+      end
+  end
+
+  def test_signature_validator_interprets_keyword_arguments_even_if_a_block_is_provided
+      FlexMock.use do |mock|
+        mock.should_receive(:m).with_signature(required_arguments: 1, optional_keyword_arguments: [:b])
+        mock.m(10, b: 10) { }
+      end
+  end
+
+  def test_signature_validator_does_not_interpret_a_proc_as_positional_argument_if_keyword_arguments_are_expected
+      FlexMock.use do |mock|
+        mock.should_receive(:m).with_signature(required_arguments: 1, required_keyword_arguments: [:b])
+        mock.m(10, b: 10) { }
+        assert_mock_failure(check_failed_error, message: /in mock 'unknown': m\(\*args\) expects at least 1 positional arguments but got only 0/, line: __LINE__+1) do
+          mock.m(b: 10) { }
+        end
+      end
+  end
+
+  def test_signature_validator_does_accept_both_a_hash_and_a_proc_as_positional_arguments
+      FlexMock.use do |mock|
+        mock.should_receive(:m).with_signature(required_arguments: 3)
+        mock.m(10, Hash[b: 10], proc {})
+      end
+  end
+
+  def test_signature_validator_does_not_accept_a_lone_hash_as_positional_argument_if_there_are_required_keyword_arguments
+      FlexMock.use do |mock|
+        mock.should_receive(:m).
+          with_signature(required_keyword_arguments: [:b], required_arguments: 1)
+        assert_mock_failure(check_failed_error, message: /in mock 'unknown': m\(\*args\) expects at least 1 positional arguments but got only 0/, line: __LINE__+1) do
+          mock.m(Hash.new)
+        end
+        assert_mock_failure(check_failed_error, message: /in mock 'unknown': m\(\*args\) expects at least 1 positional arguments but got only 0/, line: __LINE__+1) do
+          mock.m(b: 10)
+        end
+      end
+  end
+
+  def test_with_signature_matching_sets_up_the_signature_predicate_based_on_the_provided_instance_method_positional_arguments
+    k = Class.new { def m(req_a, req_b, opt_c = 10); end }
+    FlexMock.use do |mock|
+      e = mock.should_receive(:m).with_signature_matching(k.instance_method(:m))
+      e = e.instance_variable_get(:@expectations).first
+      validator = e.instance_variable_get(:@signature_validator)
+      assert_equal 2, validator.required_arguments
+      assert_equal 1, validator.optional_arguments
+      refute validator.splat?
+      assert_equal Set.new, validator.required_keyword_arguments
+      assert_equal Set.new, validator.optional_keyword_arguments
+      assert_equal false, validator.keyword_splat?
+    end
+  end
+
+  def test_with_signature_matching_sets_up_the_signature_predicate_based_on_the_provided_instance_method_splat
+    k = Class.new { def m(req_a, req_b, opt_c = 10, *splat); end }
+    FlexMock.use do |mock|
+      e = mock.should_receive(:m).with_signature_matching(k.instance_method(:m))
+      e = e.instance_variable_get(:@expectations).first
+      validator = e.instance_variable_get(:@signature_validator)
+      assert_equal 2, validator.required_arguments
+      assert_equal 1, validator.optional_arguments
+      assert validator.splat?
+      assert_equal Set.new, validator.required_keyword_arguments
+      assert_equal Set.new, validator.optional_keyword_arguments
+      assert_equal false, validator.keyword_splat?
+    end
+  end
+
+  def test_signature_validator_from_instance_method_raises_if_the_method_description_contains_an_unknown_argument_type
+    mock = flexmock(parameters: [[:unknown]])
+    error = assert_raises(ArgumentError) do
+      FlexMock::SignatureValidator.from_instance_method(flexmock, mock)
+    end
+    assert_equal "cannot interpret parameter type unknown", error.message
+  end
+
+  def test_private_Object_methods_can_be_mocked
+    FlexMock.use do |m|
+      m.should_receive(:warn).returns(1)
+      assert_equal 1, m.warn
+      assert_equal 1, m.send(:warn)
+    end
+  end
+
   private
 
   def assert_operator(op, &block)
@@ -1203,3 +1480,8 @@ class TestFlexMockArgTypesDontLeak < Minitest::Test
     assert_match(/\bany\b/, ex.message, "Error message should mention 'any'")
   end
 end
+
+if Gem::Version.new(RUBY_VERSION) >= Gem::Version.new('2.1')
+    require_relative 'should_receive_ruby21plus'
+end
+
